/************************************************************************/

/*
 *  DTFEDC.H - dtF Standard Error Description Codes / Common File
 *  (C) Copyright 1990-1993 by sLAB Banzhaf & Lang GdbR.
 *  All Rights Reserved.
 */

/************************************************************************/

#ifndef DTFEDC_H
#define DTFEDC_H

/************************************************************************/


/*----------------------------------------------------------------------*/

/* Error groups as provided to the caller.
 */

#define dtFgrpFatal         0x1000 /* Fatal internal system error */
#define dtFgrpToken         0x1010 /* User-level language token error */
#define dtFgrpSyntax        0x1020 /* User-level syntax grammar error */
#define dtFgrpIntToken      0x1030 /* Internal language token error */
#define dtFgrpIntSyntax     0x1040 /* Internal language syntax error */
#define dtFgrpIntRuntime    0x1050 /* Internal language runtime error */
#define dtFgrpTrans         0x1060 /* Transaction error */
#define dtFgrpRelation      0x1070 /* Relation error */
#define dtFgrpAuthorization 0x1080 /* Authorization error */
#define dtFgrpRefInteger    0x1090 /* Referential integrity error */
#define dtFgrpBadIo         0x1100 /* IO problem occured */
#define dtFgrpBadMemory     0x1110 /* Memory problem occured */
#define dtFgrpBadStartup    0x1120 /* Startup problem occured */
#define dtFgrpIntegrityErr  0x1130 /* Integrity problem occured */
#define dtFgrpNetErr        0x1140 /* Network problem occured */
#define dtFgrpMathErr       0x1150 /* Decimal problem occured */
#define dtFgrpResultErr     0x1160 /* Problem with result occured */
#define dtFgrpCompatErr     0x1180 /* Compatibility error occured */
#define dtFgrpSystemRes     0x1190 /* System ressource error occured */
#define dtFgrpSaaErr        0x11a0 /* Problem with dtF saa */

/*----------------------------------------------------------------------*/

/* Error Description Codes sent by the Server
 */

/* General
 */
#define dtFedcNoErr0        0x2000 /* 'No error' */
#define dtFedcNoDesc0       0x2001 /* 'Error not further described' */
#define dtFedcNoFeature     0x2002 /* 'Unimplemented feature used' */
#define dtFedcBadCmd0       0x2003 /* 'Bad internal cmd found' */
#define dtFedcErrLock0      0x2008 /* 'Requested resources are locked' */
#define dtFedcNoDatabase1   0x2009 /* 'File %1 is no valid database' */
#define dtFedcTaSysEnded0   0x200d /* 'TA aborted during deadlock' */
#define dtFedcNoAccount0    0x200e /* 'Account Failed, PW or Name wrong' */
#define dtFedcTaEnded0      0x200f /* 'Pbid no longer valid' */

/* Token
 */
#define dtFedcEofReached0   0x2110 /* 'End of req reached too early' */
#define dtFedcEofExp1       0x2111 /* 'Found %0, expected end of req' */
#define dtFedcKwExp2        0x2112 /* 'Found %0, expected %1' */
#define dtFedcExpCmd1       0x2113 /* 'Found %0, expected command' */
#define dtFedcExpTbl1       0x2114 /* 'Found %0, expected table name' */
#define dtFedcExpCol1       0x2115 /* 'Found %0, expected column name' */
#define dtFedcExpType1      0x2116 /* 'Found %0, expected type name' */
#define dtFedcExpUsr1       0x2117 /* 'Found %0, expected user name' */
#define dtFedcExpPsw1       0x2118 /* 'Found %0, expected password' */
#define dtFedcExpAcc1       0x2119 /* 'Found %0, expected acc. rights' */
#define dtFedcExpVal1       0x211a /* 'Found %0, expected user value' */
#define dtFedcExpCond1      0x211b /* 'Found %0, expected condition' */
#define dtFedcExpFactor1    0x211c /* 'Found %0, expected factor' */
#define dtFedcExpNumber1    0x211d /* 'Found %0, expected number' */

/* Internal Runtime
 */
#define dtFedcNoMem0        0x2200 /* 'Server main memory overflow' */
#define dtFedcOverflow0     0x2201 /* 'Table or stack overflow' */
#define dtFedcBadNewRec0    0x2202 /* 'Record creation error' */
#define dtFedcBadNewCol1    0x2203 /* 'Column %0 creation error' */
#define dtFedcBadNewIdx1    0x2204 /* 'Index on Col %0 creation error' */
#define dtFedcBadAcc1       0x2205 /* 'Access for %0 creating error' */
#define dtFedcLockFull0     0x2210 /* 'Lock Table overflow' */
#define dtFedcTAtblFull0    0x2211 /* 'TA Table overflow' */
#define dtFedcDBFull0       0x2212 /* 'Database limit reached' */
#define dtFedcScanFull0     0x2213 /* 'Search condition limit reached' */
#define dtFedcRAstackFull0  0x2214 /* 'Request is too complex' */
#define dtFedcColumnsFull0  0x2215 /* 'Too many columns selected' */
#define dtFedcProjFull0     0x2216 /* 'Too many expr in projections' */

/* Runtime
 */
#define dtFedcParam0        0x2300 /* 'Parameter incompatibility' */
#define dtFedcBadVal1       0x2301 /* 'Bad number of values at func %1' */
#define dtFedcBadTypeX1     0x2302 /* 'Parameter %1 has bad type' */
#define dtFedcBadFunc1      0x2303 /* 'Function %1 not available' */
#define dtFedcDivByZero0    0x2304 /* 'Division by zero error' */
#define dtFedcConvtRange1   0x2305 /* 'Value %1 has bad range'*/
#define dtFedcConvtBad1     0x2306 /* 'Value %1 illegal characters'*/
#define dtFedcConvtSigned1  0x2307 /* 'Value %1 signed mismatch'*/
#define dtFedcParam1        0x2308 /* 'Param.incompatibility for %1' */
#define dtFedcNoConvert1    0x2309 /* 'Value %1 can not be impl. converted */
#define dtFedcBadSumAgg     0x230a /* 'Bad usage of SUM or AGG function */
#define dtFedcBadParam1     0x230b /* 'Unknown parameter %1' */
#define dtFedcBadTyRange1   0x230c /* '%1 is out of range for datatype' */

/* Transactions
 */
#define dtFedcNoNestT0      0x2400 /* 'No nested trans. allowed' */
#define dtFedcNoT0          0x2401 /* 'No transaction started' */

/* Relation
 */
#define dtFedcNoTbl1        0x2500 /* 'Table %0 does not exist' */
#define dtFedcBadCol1       0x2501 /* 'Column %0 does not exist' */
#define dtFedcNoIdx1        0x2502 /* 'Index %0 does not exist' */
#define dtFedcNoUsr1        0x2503 /* 'User %0 does not exist' */

#define dtFedcExsTbl1       0x2510 /* 'Table %0 exists' */
#define dtFedcExsCol1       0x2511 /* 'Column %0 exists' */
#define dtFedcExsIdx1       0x2512 /* 'Index %0 exists' */

#define dtFedcTooManyTbls0  0x2520 /* 'Too many tables' */
#define dtFedcTooManyCols0  0x2521 /* 'Too many columns for table' */
#define dtFedcTooManyUsr0   0x2522 /* 'Too many users defined' */

#define dtFedcBadVal0       0x2530 /* 'Bad number of Values' */
#define dtFedcBadColTypes2  0x2531 /* 'Columns %0, %1 have inc. types' */
#define dtFedcBadType1      0x2532 /* 'Value %0 has incompatible type' */

#define dtFedcDblTblVar1    0x2540 /* '%0 represents several tables' */
#define dtFedcDoubleCol1    0x2541 /* 'Found double column %0' */
#define dtFedcTooLarge1     0x2542 /* 'Itentifier %0' is too large' */

#define dtFedcBadJoin1      0x25f0 /* 'No join for %1 supported' */
#define dtFedcBadTbl        0x25f1 /* 'Tables do not match' */

/* Authorization
 */
#define dtFedcNoAccess0     0x2600 /* 'Access denied' */
#define dtFedcNoAccRight1   0x2601 /* 'No %0 Access.' */
#define dtFedcNoCreat1      0x2602 /* 'User is not creator of %0' */

/* Referential integrity
 */
#define dtFedcBadNull1      0x2700 /* 'NULL value not allowed for col %0' */
#define dtFedcBadUnique1    0x2701 /* 'UNIQUE value exists for col %0' */

/* Server Memory Errors
 */
#define dtFedcSErrMem0       0x2f00 /* 'Memory error.' */
#define dtFedcSErrVMem0      0x2f01 /* 'Virtual Memory error.' */
#define dtFedcSErrVFull0     0x2f02 /* 'Virtual Memory Init error.' */
#define dtFedcSErrVMemFull0  0x2f03 /* 'Virtual Memory exhausted.' */
#define dtFedcSErrVMemSlot0  0x2f04 /* 'Virtual Memory slots exceeded.' */
#define dtFedcSErrNoManager1 0x2f05 /* 'Memory Manager '%0' not found.' */
#define dtFedcSErrManager1   0x2f06 /* 'Memory Manager error '%0'.' */

/*----------------------------------------------------------------------*/

/* Error Description Codes of the Client and Initialization
 */

/* I/O Problems
 */
#define dtFedcErrIO0        0x3001 /* 'General IO error.' */
#define dtFedcErrNoFile1    0x3002 /* 'File %0 not found.' */
#define dtFedcErrAccFile1   0x3003 /* 'File %0 access error.' */
#define dtFedcErrCrtFile1   0x3004 /* 'File %0 exists.' */
#define dtFedcErrClsFile0   0x3005 /* 'Closing file error.' */
#define dtFedcErrWrtFile0   0x3006 /* 'Writing file error.' */
#define dtFedcErrReaFile0   0x3007 /* 'Reading file error.' */

/* Startup and Setupfile Problems
 */
#define dtFedcErrSyntax2    0x3200 /* 'Found '%0' expected '%1'.' */
#define dtFedcNoSection1    0x3201 /* 'Section %0 not found.' */
#define dtFedcEofTooSoon0   0x3202 /* 'EOF reached too early.' */
#define dtFedcNoParam1      0x3203 /* 'Unknown parameter '%0'.' */
#define dtFedcNoSwitch1     0x3204 /* 'Unknown switch '%0'.' */
#define dtFedcBadSecParam2  0x3205 /* 'Parameter '%0' bad for sec '%1'.' */

/* Integrity
 */
#define dtFedcErrInteg0     0x3300 /* 'General Integrity Error.' */
#define dtFedcBadSize0      0x3301 /* 'Integrity Bad Filesize.' */
#define dtFedcBadCheck0     0x3302 /* 'Integrity Bad Checksum.' */
#define dtFedcBadCounter0   0x3303 /* 'Integrity of page counter.' */
#define dtFedcBadRecords0   0x3304 /* 'Integrity of records.' */
#define dtFedcDBinUse1      0x3305 /* 'Database [%0] is in use.'*/
#define dtFedcTooManyPg0    0x3310 /* 'Cannot recover too many bad pages.'*/
#define dtFedcIntInteg0     0x3311 /* 'Cannot recover internal integrity.'*/
#define dtFedcBadCluster0   0x3312 /* 'Cannot recover bad cluster.'*/
#define dtFedcBadIndex1     0x3320 /* 'Index '%0' dropped rerun recover.'*/
#define dtFedcBadRecord1    0x3321 /* 'Rec. in rel. '%0' dropped, rerun rec.'*/

/* Net-problems
 */
#define dtFedcErrNet0       0x3400 /* 'General Net Error.' */
#define dtFedcErrNoNet0     0x3401 /* 'Net System not found.' */
#define dtFedcErrComm2      0x3402 /* 'Communication error %1 during %2.' */
#define dtFedcErrSync1      0x3403 /* 'Communication sync error %1.' */

/* Math
 */
#define dtFedcErrMath0      0x3500 /* 'General Math Error.' */
#define dtFedcDDivByZero0   0x3501 /* 'Divison by Zero.' */
#define dtFedcStrongType2   0x3502 /* 'Type incompatibility '%0', '%1' */
#define dtFedcDecStrForm1   0x3503 /* 'Cannot transform string '%0'.' */

/* Level2 result
 */
#define dtFedcErrResult0    0x3600 /* 'General runtime error.' */
#define dtFedcErrNoResult0  0x3601 /* 'No result data available.' */
#define dtFedcErrEmptyTbl0  0x3602 /* 'Empty table received as result.' */
#define dtFedcErrCrsrRange0 0x3603 /* 'Cursor range overflow.' */
#define dtFedcErrColRange0  0x3604 /* 'Column range overflow.' */
#define dtFedcErrBufOver1   0x3605 /* 'Buffer overflow for field '%0'.' */
#define dtFedcErrValRange0  0x3606 /* 'Value out of range.' */
#define dtFedcErrNoRecs0    0x3607 /* 'No records performed during request.' */

/* Compatibility
 */
#define dtFedcErrCompat0    0x3800 /* 'General runtime error.' */
#define dtFedcErrBadVer2    0x3801 /* 'Version '%0.' not compat with '%1'.' */
#define dtFedcErrBadType0   0x3802 /* 'Trial is not compat with official'. */

/* System ressource
 */
#define dtFedcErrSys0       0x3900 /* 'General system ressource error.' */
#define dtFedcErrSys1       0x3901 /* 'System ressource error '%1'.' */
#define dtFedcErrResFull0   0x3902 /* 'Result overflow' */

/* dtF saa
 */
#define dtFedcErrSaa0       0x3a00 /* 'General SAA error occured.' */
#define dtFedcSaaObjFull0   0x3a01 /* 'No more objects can be created.'  */
#define dtFedcSaaPropFull0  0x3a02 /* 'No more properties can be created.' */
#define dtFedcSaaNoProp2    0x3a03 /* 'Object '%0' has no property '%1'.' */
#define dtFedcSaaNoObj1     0x3a04 /* 'Object '%0' does not exist.' */
#define dtFedcSaaBadParent2 0x3a05 /* 'Class '%0' has no parent '%1' .' */

/* Client Memory Errors
 */
#define dtFedcCErrMem0       0x3f00 /* 'Memory error.' */
#define dtFedcCErrVMem0      0x3f01 /* 'Virtual Memory error.' */
#define dtFedcCErrVFull0     0x3f02 /* 'Virtual Memory Init error.' */
#define dtFedcCErrVMemFull0  0x3f03 /* 'Virtual Memory exhausted.' */
#define dtFedcCErrVMemSlot0  0x3f04 /* 'Virtual Memory slots exceeded.' */
#define dtFedcCErrNoManager1 0x3f05 /* 'Memory Manager '%0' not found.' */
#define dtFedcCErrManager1   0x3f06 /* 'Memory Manager error '%0'.' */

/************************************************************************/

#endif

/************************************************************************/
