/************************************************************************/

/*
 *  DTFLVL1.H - dtF Level 1 Application Development / Common File
 *  (C) Copyright 1990-1993 by sLAB Banzhaf & Lang GdbR.
 *  All Rights Reserved.
 */

/************************************************************************/

#ifndef DTFLVL1_H
#define DTFLVL1_H

/************************************************************************/

/* Check whether C++ is used
 */

#if ( _CPLUSPLUS | _cplusplus | __CPLUSPLUS | __cplusplus )
extern "C" {
#endif

/************************************************************************/

/* Standard Types and Constants
 */

/*----------------------------------------------------------------------*/

/* Basic Types and Constants Dependent on Platform
 */

#if DTFMSDOS | DTFOS2 | DTFWINDOWS

typedef unsigned char Byte;

#if DTF32B
typedef unsigned short Word;
#else
typedef unsigned int Word;
#endif

typedef unsigned long LongWord;

#if DTF32B
typedef signed char SignedByte;
#else
typedef char SignedByte;
#endif

#if DTF32B
typedef short SignedWord;
#else
typedef int SignedWord;
#endif

typedef long SignedLongWord;
typedef double Real;

#define ByteNoBits 0
#define ByteAllBits 0xff
#define ByteRangeDiv2 0x80
#define ByteBits 8
#define Log2ByteBits 3

#define WordNoBits 0x0000
#define WordAllBits 0xffff
#define WordRangeDiv2 0x8000
#define WordBits 16
#define Log2WordBits 4

#define LongWordNoBits 0L
#define LongWordAllBits 0xffffffffL
#define LongWordRangeDiv2 0x80000000L
#define LongWordBits 32
#define Log2LongWordBits 5

#define RealNoBits 0
#define RealAllBits 1.7976931348623158e+308
#define RealRangeDiv2 0
#define RealBytes 11
#define RealBits 88
#define Log2RealBits 7

#endif

#if DTFMAC

#define Byte M_Byte
#define SignedByte M_SignedByte

typedef unsigned char Byte;
typedef unsigned short Word;
typedef unsigned long LongWord;
typedef char SignedByte;
typedef short SignedWord;
typedef long SignedLongWord;

#if THINKC
typedef short double Real;
#else
typedef double Real;			/* 8 Byte for MPW C, without mc68881 */
#endif


#define ByteNoBits 0x00
#define ByteAllBits 0xff
#define ByteRangeDiv2 0x80
#define ByteBits 8
#define Log2ByteBits 3

#define WordNoBits 0x0000
#define WordAllBits 0xffff
#define WordRangeDiv2 0x8000
#define WordBits 16
#define Log2WordBits 4

#define LongWordNoBits 0L
#define LongWordAllBits 0xffffffffL
#define LongWordRangeDiv2 0x80000000L
#define LongWordBits 32
#define Log2LongWordBits 5

#define RealNoBits 0.0
#define RealAllBits 1.7976931348623157e+300
#define RealRangeDiv2 0
#define RealBytes 10
#define RealBits 80
#define Log2RealBits 7

#if THINKC
#undef nil
#undef NIL
#endif

/*
#undef NULL
#define NULL ((void *)0)
*/

#endif

/*----------------------------------------------------------------------*/

/* Basic Types and Constants Independent of Platform
 */

#if ! DTFMAC
#define Boolean dtFboolean
#endif

#if ! ( DTF32B )
typedef enum { False, True } dtFboolean;
#else
typedef Word dtFboolean;
#define False ((dtFboolean)WordNoBits)
#define True  ((dtFboolean)(WordNoBits + 1))
#endif

#define IsTrue(x) (((dtFboolean)x)!=False)
#define IsFalse(x) (((dtFboolean)x)==False)

#ifndef max
#define max(a,b)  (((a) > (b)) ? (a) : (b))
#endif

#ifndef min
#define min(a,b)  (((a) < (b)) ? (a) : (b))
#endif

/*----------------------------------------------------------------------*/
/* Sizes and Types of Date, Time 
 */

typedef SignedLongWord dtFtime, *dtFtimePtr;
typedef SignedLongWord dtFdate, *dtFdatePtr;

#define dtFextDateBytes 11
#define dtFextTimeBytes  9

#define HugeWordWords   4
#define HugeWordBytes   8
#define HugeWordDigits 16

typedef struct {
    Word w[HugeWordWords];
} HugeWord, *HugeWordPtr;

/*----------------------------------------------------------------------*/

typedef void *VoidPtr;

/*----------------------------------------------------------------------*/

typedef Byte *BytePtr;
typedef Word *WordPtr;
typedef LongWord *LongWordPtr;
typedef SignedByte *SignedBytePtr;
typedef SignedWord *SignedWordPtr;
typedef SignedLongWord *SignedLongWordPtr;
typedef Real *RealPtr;

/*----------------------------------------------------------------------*/

typedef Byte *MemPtr;
typedef MemPtr *MemPtrPtr;

#define BadMemPtr NULL

/*----------------------------------------------------------------------*/

typedef LongWord MemIdRange;

/*----------------------------------------------------------------------*/
/* Sizes and Types for Decimal Arithmetic
 */

#define dtFextDecimalBytes  19
#define dtFextDecimalDigits 16

#define NOPREC    0xFF
#define RESULTERR 0xFF

typedef struct {
    HugeWord value;
    Byte     prec;
} dtFdecimal, *dtFdecimalPtr;

/************************************************************************/

/* Internal Error Codes (16Bit values)
 */

#if DTF32B

typedef signed short ErrType;
  
#define ErrOK            0x00
#define ErrBAD           0x01
#define ErrFATAL         0x02
#define ErrOther         0x03
#define ErrBadId         0x04
#define ErrLock          0x05
#define ErrNoSeg         0x06
#define ErrNoPage        0x07
#define ErrNoBuffer      0x08
#define ErrIO            0x09
#define ErrFull          0x0a
#define ErrNoFile        0x0b
#define ErrRange         0x0c
#define ErrFile          0x0d
#define ErrMemory        0x0e
#define ErrIntegrity     0x0f
#define ErrNoScan        0x10
#define ErrNoMoreRecords 0x11
#define ErrBufferFull    0x12
#define ErrExists        0x13
#define ErrDoesNotExist  0x14
#define ErrServer        0x15
#define ErrClient        0x16
#define ErrSync          0x17
#define ErrNet           0x18
#define ErrStopped       0x19
#define ErrPassword      0x1a
#define ErrAccess        0x1b
#define ErrDivByZero     0x1c
#define ErrConversion    0x1d
#define ErrRessource     0x1e

#else

typedef enum {
   ErrOK,           ErrBAD,           ErrFATAL,      ErrOther,  
   ErrBadId,        ErrLock,          ErrNoSeg,      ErrNoPage,
   ErrNoBuffer,     ErrIO,            ErrFull,       ErrNoFile,
   ErrRange,        ErrFile,          ErrMemory,     ErrIntegrity,
   ErrNoScan,       ErrNoMoreRecords, ErrBufferFull, ErrExists,
   ErrDoesNotExist, ErrServer,        ErrClient,     ErrSync,
   ErrNet,          ErrStopped,       ErrPassword,   ErrAccess,
   ErrDivByZero
} ErrType;

#endif

/*----------------------------------------------------------------------*/

/* Current Release Constants
 */

extern MemPtr DTFFAR dtFversion;
extern MemPtr DTFFAR dtFcopyright;
extern MemPtr DTFFAR dtFsystem;

#define dtFtypeTrial    0x0010
#define dtFtypeInternal 0x0020
#define dtFtypeOfficial 0x0080

#define dtFversion000   0x0000
#define dtFversion100   0x0100
#define dtFversion110   0x0110
#define dtFversion113   0x0113
#define dtFversion120   0x0120
#define dtFversion130   0x0130

extern Word DTFFAR dtFtypeNo;
extern Word DTFFAR dtFversionNo;   /* dtF version number 0x0130 -> V1.30  */

/************************************************************************/

/* Function Prototypes
 */

/*----------------------------------------------------------------------*/

/* General Functions
 */

/*----------------------------------------------------------------------*/

/* Generic Memory Handling
 */

extern MemIdRange DTFFAR DTFCDCL dtFmemAlloc(LongWord);
extern ErrType DTFFAR DTFCDCL dtFmemRealloc(MemIdRange, Word);
extern ErrType DTFFAR DTFCDCL dtFmemRelease(MemIdRange);
extern MemPtr DTFFAR DTFCDCL dtFmemLock(MemIdRange);
extern ErrType DTFFAR DTFCDCL dtFmemUnlock(MemIdRange);
extern LongWord DTFFAR DTFCDCL dtFqueryFreeMemory(void);

/************************************************************************/

/* Initializationfile handling
 */

typedef enum {
   dtFinitApps, dtFinitUser
} dtFinitRange;                   /* possible initialization situations */

extern MemPtr DTFFAR DTFCDCL SCgetInitString(dtFinitRange type,
   MemPtr fname, MemPtr parameter);

/************************************************************************/

/* Communication and Named Pipe
 */

#define dtFpipeName "\\PIPE\\dtF.pip"

/*----------------------------------------------------------------------*/

/* Different Server Priority Levels
 */

typedef enum {
   dtFlowPrio, dtFmediumPrio, dtFhighPrio, dtFsystemPrio
} dtFprioRange;

/************************************************************************/

/* Definitions for Client/Server Data Exchange
 */

/*----------------------------------------------------------------------*/

/* Request and Result Format Identifiers
 */

#define SCrequests            3       /* # of known request formats     */
#define SCreqUndefined   0x0000       /* Undefined Request              */
#define SCreqNulSQL      0x0001       /* NUL SQL                        */
#define SCreqRelAlgebra  0x0002       /* Relational Algebra             */

#define dtFresNone        0x0000      /* No data result provided        */
#define dtFresError       0x0001      /* Error description provided     */
#define dtFresRecEnum     0x0002      /* Direct record enumeration      */

/*----------------------------------------------------------------------*/

/* Request and Result Handling Definitions
 */

#define dtFpageSize 4096

typedef MemPtr SCresPagePtr;          /* DO NOT USE, may change         */
typedef SCresPagePtr dtFresPtr;       /* Result page pointer            */

typedef Word dtFrequestHandle;        /* Request handle identifier      */

typedef Word dtFpbid,                 /* Mailbox identifier at server   */
             *dtFpbidPtr;             /* Pointer to dtfPbid             */

/*----------------------------------------------------------------------*/

/* Different Request Status Values 
 */

typedef enum {
   dtFreqUnused,                      /* client memory overflow         */
   dtFreqSent,                        /* req. sent, accepted by server  */
   dtFreqRunning,                     /* server actually works on req.  */
   dtFreqFinished,                    /* req. is successfully finished  */
   dtFreqRejected,                    /* error occured, r. not accepted */
   dtFreqTAEnded                      /* TA ended, pbid no longer valid */
} dtFrequestStatus;

/************************************************************************/

/* Constants and Definitions for Client Cursor Functionality
 */

/*----------------------------------------------------------------------*/

/* Global Error Variable
 */

extern ErrType dtFerr;                /* Error variable of the client   */

/*----------------------------------------------------------------------*/

/* Independent Constants for Cursor Purpose
 */

#define dtFbadCursor LongWordNoBits         /* Cursor definition failed */
#define dtFnoRecords LongWordNoBits         /* Opened cursor is empty   */

/*----------------------------------------------------------------------*/

/* Types for Cursor Purpose
 */

typedef LongWord dtFcursorSize;       /* # of records within one cursor */
typedef LongWord dtFcursorHandle;     /* Handle of a defined cursor     */

/*----------------------------------------------------------------------*/

/* Different Cursor Types
 */

typedef enum {
   dtFrawCursor,                      /* Raw cursor, size is 1 record   */
   dtFndxCursor,                      /* Raw cursor, indexed            */
   dtFunused1, dtFunused2             /* RESERVED                       */
} dtFcursorType;

/*----------------------------------------------------------------------*/

/* Different Cursor Movements
 */

typedef enum {
   dtFmvRecRel,                       /* Move to absolute record pos.   */
   dtFmvRecAbs,                       /* Move relative by # records     */
   dtFmvPageRel,                      /* Move to absolute page number   */
   dtFmvPageAbs                       /* Move relative by # pages       */
} dtFmoveType;

typedef SignedLongWord dtFcursorRange;/* Range to move within cursor    */

/************************************************************************/

/* Constants and Definitions for Record and Field Functionality
 */

/************************************************************************/

/* Record and Field Types
 */

#define dtFmaxFieldLen           250  /* Standard fields max. length    */
#define dtFmaxRelations          120  /* Max. # of relations of one db. */
#define dtFmaxFieldsPerRel       128  /* Max. # of fields of one rel.   */

#define dtFcolInfoSize 64              /* SCresRecEnum: max # of res.col */
#define dtFnameSize    24              /* SCresRecEnum: name string size */
#define dtFpwSize      16              /* Length of password */

typedef Byte dtFfieldRange;           /* Type for # of fields of 1 rec. */

typedef Word dtFrecordLen,            /* Record length type             */
             *dtFrecordLenPtr;        /* Pointer for record length type */

typedef Byte dtFfieldLen,             /* Field length type              */
             *dtFfieldLenPtr;         /* Pointer for field length type  */

/*----------------------------------------------------------------------*/

/* Constants Describing User Column Types
 */


#if ! ( DTF32B )
typedef enum {
   dtFcolTypeDecimal   = 0x0000,         /* Decimal type */
   dtFcolTypeByte      = 0x0f00,         /* Byte type */
   dtFcolTypeWord,                       /* Word type */
   dtFcolTypeLongWord,                   /* LongWord type */
   dtFcolTypeChar,                       /* Character type */
   dtFcolTypeShort,                      /* ShortInteger type */
   dtFcolTypeLong,                       /* LongInteger type */
   dtFcolTypeReal,                       /* Real Type */
   dtFcolTypeShortString,                /* ShortString type */
   dtFcolTypeHugeString,                 /* HugeString type */
   dtFcolTypeDate,                       /* Date type */
   dtFcolTypeTime,                       /* Time type */
   dtFcolTypeUndefined = 0x0fff          /* Undefined type */
} dtFcolTypeRange, *dtFcolTypeRangePtr;
#else
typedef Word dtFcolTypeRange, *dtFcolTypeRangePtr;
#define dtFcolTypeDecimal     0x0000         /* Decimal type */
#define dtFcolTypeByte        0x0f00         /* Byte type */
#define dtFcolTypeWord        0x0f01         /* Word type */
#define dtFcolTypeLongWord    0x0f02         /* LongWord type */
#define dtFcolTypeChar        0x0f03         /* Character type */
#define dtFcolTypeShort       0x0f04         /* ShortInteger type */
#define dtFcolTypeLong        0x0f05         /* LongInteger type */
#define dtFcolTypeReal        0x0f06         /* Real Type */
#define dtFcolTypeShortString 0x0f07         /* ShortString type */
#define dtFcolTypeHugeString  0x0f08         /* HugeString type */
#define dtFcolTypeDate        0x0f09         /* Date type */
#define dtFcolTypeTime        0x0f0a         /* Time type */
#define dtFcolTypeUndefined   0x0fff         /* Undefined type */
#endif


#define dtFcolTypes 13

#define dtFdecimalRanges 256

#define dtFdecimalPre(type)  ((Byte)(((Byte)type >> 4))+1)
#define dtFdecimalPost(type) ((Byte)((Byte)type & 0x0f))

#define dtFisExtendedDataType(type)   ((type & 0xf000) != 0)
#define dtFgetBaseDataType(type)       (type & 0x0fff)

/************************************************************************/

/* The following definitions are not for public usage by developers.
 */

/*----------------------------------------------------------------------*/

/* Private Low Level Function Prototypes of the Client, DO NOT USE
 */

extern ErrType DTFFAR DTFCDCL CUinitDtf(MemPtr);
extern ErrType DTFFAR DTFCDCL CUendWorkDtf(void);

extern dtFrequestHandle DTFFAR DTFCDCL dtFsend(dtFpbidPtr, MemPtr);
extern dtFresPtr DTFFAR DTFCDCL dtFgetErrorPage(dtFrequestHandle);

/************************************************************************/

/* Public Function Prototypes For Client Interface
 */

/*----------------------------------------------------------------------*/

/* Module Initialization and Endwork
 */

extern ErrType DTFFAR DTFCDCL dtFinit(MemPtr);
extern ErrType DTFFAR DTFCDCL dtFendWork();
extern ErrType DTFFAR DTFCDCL dtFconnectServer(dtFpbidPtr, MemPtr, MemPtr);

/*----------------------------------------------------------------------*/

/* Request Send, Status, Wait and Drop Functions 
 */

#define dtFasyncRequest(pbid, request) dtFsend(pbid, request)

#define dtFsyncRequest(pbid, request, time) \
   dtFwaitForResult(dtFsend(pbid, request), time)

extern dtFrequestStatus DTFFAR DTFCDCL dtFstatus(dtFrequestHandle,
   dtFpbidPtr);
extern dtFrequestHandle DTFFAR DTFCDCL dtFwaitForResult(
   dtFrequestHandle, SignedLongWord);
extern ErrType DTFFAR DTFCDCL dtFdropResult(dtFrequestHandle);
extern Word DTFFAR DTFCDCL dtFresultFormat(dtFrequestHandle);
extern LongWord DTFFAR DTFCDCL dtFperformedRecords(dtFrequestHandle);

/*----------------------------------------------------------------------*/

/* Header Page Functions
 */

extern dtFresPtr DTFFAR DTFCDCL dtFlockHeader(dtFrequestHandle);
extern ErrType DTFFAR DTFCDCL dtFunlockHeader(dtFrequestHandle);
extern dtFcursorSize DTFFAR DTFCDCL dtFgetRecords(dtFresPtr);
extern dtFfieldRange DTFFAR DTFCDCL dtFgetColumns(dtFresPtr);
extern MemPtr DTFFAR DTFCDCL dtFgetRelName(dtFresPtr,dtFfieldRange);
extern MemPtr DTFFAR DTFCDCL dtFgetColName(dtFresPtr,dtFfieldRange);
extern dtFcolTypeRange DTFFAR DTFCDCL dtFgetColType(dtFresPtr,dtFfieldRange);
extern Word DTFFAR DTFCDCL dtFgetExtColType(dtFresPtr,dtFfieldRange);
extern dtFboolean DTFFAR DTFCDCL dtFisSorted(dtFresPtr);

/*----------------------------------------------------------------------*/

/* Cursor Functions
 */

extern dtFcursorHandle DTFFAR DTFCDCL dtFdefineCursor(dtFcursorSize,
   dtFcursorType);
extern ErrType DTFFAR DTFCDCL dtFdropCursor(dtFcursorHandle);
extern MemPtr DTFFAR DTFCDCL dtFopenCursor(dtFcursorHandle,dtFrequestHandle);
extern ErrType DTFFAR DTFCDCL dtFcloseCursor(dtFcursorHandle);
extern MemPtr DTFFAR DTFCDCL dtFmoveCursor(dtFcursorHandle, dtFmoveType, 
   dtFcursorRange);
extern MemPtr DTFFAR DTFCDCL dtFmoveCursor(dtFcursorHandle, dtFmoveType, 
   dtFcursorRange);
extern dtFcursorRange DTFFAR DTFCDCL dtFcrsrPos(dtFcursorHandle);


/************************************************************************/

/* Several Help Functions
 */

/*----------------------------------------------------------------------*/

/* Error Handling 
 */

extern MemPtr DTFFAR DTFCDCL dtFgetError(ErrType);
extern ErrType DTFFAR DTFCDCL dtFgetErrorMsg(MemPtr, MemPtr,
   dtFrequestHandle, MemPtr);
extern ErrType DTFFAR DTFCDCL dtFgetErrorMsgX(MemPtr, MemPtr, WordPtr,
   MemPtrPtr, WordPtr, dtFrequestHandle, MemPtr);
extern dtFboolean DTFFAR DTFCDCL dtFaccountFailed(dtFrequestHandle);

/*----------------------------------------------------------------------*/

/* Retrieving Field Values 
 */

extern dtFfieldRange DTFFAR DTFCDCL dtFscanCrsr(dtFresPtr, MemPtr, ...);
extern dtFfieldRange DTFFAR DTFCDCL dtFscanCrsrL(dtFresPtr, MemPtr, ...);
extern dtFfieldRange DTFFAR DTFCDCL dtFscanCrsrX(dtFresPtr, MemPtr, ...);
extern MemPtr DTFFAR DTFCDCL dtFgetField(
   dtFresPtr, MemPtr, dtFfieldRange, MemPtr, dtFfieldLenPtr);

extern ErrType DTFFAR DTFCDCL dtFputValueFromFile(dtFpbidPtr,
  dtFrequestHandle, dtFcursorRange, dtFfieldRange, MemPtr);
extern ErrType DTFFAR DTFCDCL dtFgetValueToFile(dtFpbidPtr,
  dtFrequestHandle, dtFcursorRange, dtFfieldRange, MemPtr);

/*----------------------------------------------------------------------*/

/* Data Conversion Routines 
 */

extern MemPtr DTFFAR DTFCDCL dtFconvtExtInt(dtFcolTypeRange, MemPtr);
extern MemPtr DTFFAR DTFCDCL dtFconvtExtIntX(dtFcolTypeRange, MemPtr, MemPtr);
extern MemPtr DTFFAR DTFCDCL dtFconvtIntExt(dtFcolTypeRange, MemPtr);
extern MemPtr DTFFAR DTFCDCL dtFconvtIntExtX(dtFcolTypeRange, MemPtr, MemPtr);
extern MemPtr DTFFAR DTFCDCL dtFconvtToStr(dtFcolTypeRange, MemPtr,
   MemPtr, Word, Word);

extern dtFdate DTFFAR DTFCDCL dtFconvtToDate(Word, Word, Word);
extern void DTFFAR DTFCDCL dtFconvtFromDate(dtFdate, WordPtr,
   WordPtr, WordPtr);
extern Word DTFFAR DTFCDCL dtFconvtDayOfWeek(dtFdate);
extern dtFboolean DTFFAR DTFCDCL dtFconvtIsDate(Word, Word, Word);

extern dtFtime DTFFAR DTFCDCL dtFconvtToTime(Word, Word, Word);
extern void DTFFAR DTFCDCL dtFconvtFromTime(dtFtime, WordPtr,
   WordPtr, WordPtr);
extern dtFboolean DTFFAR DTFCDCL dtFconvtIsTime(Word, Word, Word);

/************************************************************************/

/* Decimal Arithmetic Functions
 */

/*----------------------------------------------------------------------*/

/* Decimal Functions implemented as macros
 */

#define dtFdecPlus(op1, op2)        dtFdecimalPlus(NOPREC, (op1), (op2))
#define dtFdecMinus(op1, op2)       dtFdecimalMinus(NOPREC, (op1), (op2))
#define dtFdecTimes(op1, op2)       dtFdecimalTimes(NOPREC, (op1), (op2))
#define dtFdecDiv(op1, op2)         dtFdecimalDiv(NOPREC, (op1), (op2))

/*----------------------------------------------------------------------*/

/* Check for decimal error condition
 */

#define dtFdecimalIsErr(op)         ((op).prec == RESULTERR)

/*----------------------------------------------------------------------*/

/* Decimal Functions
 */

extern dtFdecimal DTFFAR DTFCDCL dtFdecAbs(dtFdecimal operand);

extern dtFdecimal DTFFAR DTFCDCL dtFdecimalPlus(
   Byte ergprec, dtFdecimal operand1, dtFdecimal operand2);
extern dtFdecimal DTFFAR DTFCDCL dtFdecimalMinus(
   Byte ergprec, dtFdecimal operand1, dtFdecimal operand2);
extern dtFdecimal DTFFAR DTFCDCL dtFdecimalTimes(
   Byte ergprec, dtFdecimal operand1, dtFdecimal operand2);
extern dtFdecimal DTFFAR DTFCDCL dtFdecimalDiv(
   Byte ergprec, dtFdecimal operand1, dtFdecimal operand2);
extern dtFdecimal DTFFAR DTFCDCL dtFdecimalNorm(Byte ergprec, 
  dtFdecimal operand);

/*----------------------------------------------------------------------*/

/* Decimal Conversion Routines
 */

extern dtFdecimal DTFFAR DTFCDCL dtFdecimalConst(MemPtr numstring);
extern MemPtr DTFFAR DTFCDCL dtFdecimalToStr(
   MemPtr strbuf, Word buflen, dtFdecimal value, Word length);
extern dtFdecimal DTFFAR DTFCDCL dtFlongToDecimal(Byte ergprec,
   SignedLongWord value);

/*----------------------------------------------------------------------*/

/* Decimal Comparison Routines
 */

extern dtFboolean DTFFAR DTFCDCL dtFdecimalEq(dtFdecimal value1,
   dtFdecimal value2);
extern dtFboolean DTFFAR DTFCDCL dtFdecimalLeq(dtFdecimal value1,
   dtFdecimal value2);
extern dtFboolean DTFFAR DTFCDCL dtFdecimalGt(dtFdecimal value1,
   dtFdecimal value2);

/************************************************************************/

#if ( _CPLUSPLUS | _cplusplus | __CPLUSPLUS | __cplusplus )
}
#endif


/************************************************************************/

#endif
