/************************************************************************/

/*
 *  DTFLVL2.H - dtF Level 2 Application Development / Common File
 *  (C) Copyright 1990-1993 by sLAB Banzhaf & Lang GdbR. 
 *  All Rights Reserved.
 */

/************************************************************************/

#ifndef DTFLVL2_H
#define DTFLVL2_H

/************************************************************************/

/* Check whether C++ is used
 */

#if ( _CPLUSPLUS | _cplusplus | __CPLUSPLUS | __cplusplus )
extern "C" {
#endif

/************************************************************************/

/* Types and Constants
 */

/*----------------------------------------------------------------------*/

/* Current Release Constant
 */

extern MemPtr DTFFAR dtF2version;
extern Word DTFFAR dtF2versionNo;

#define dtF2version000   0x0000
#define dtF2version100   0x0100
#define dtF2version110   0x0110
#define dtF2version113   0x0113
#define dtF2version120   0x0120
#define dtF2version130   0x0130

/*----------------------------------------------------------------------*/

/* Workspace Handle
 */

typedef Word dtF2wsHandle;
#define dtF2badWS WordAllBits

/*----------------------------------------------------------------------*/

/* Column Name / Column Table / Column Type Information Structure
 */

typedef struct {
   Byte table[dtFnameSize];
   Byte column[dtFnameSize];
   dtFcolTypeRange type;
   Word exttype; 
} dtF2cinfoEntry;

/*----------------------------------------------------------------------*/

/* Different Movements Types
 */

typedef enum {
   dtF2moveStart,                    /* Move from the start of the data */
   dtF2moveEnd,                      /* Move from actual poistion       */
   dtF2moveCurrent                   /* Move from the last record       */
} dtF2moveType;

/*----------------------------------------------------------------------*/

/* Function Prototype Definition for Exception Handling Functions
 */

typedef ErrType (*dtF2exceptionPtr)(Word code, MemPtr group, MemPtr desc,
    MemPtr sqlstr, Word errpos);

/************************************************************************/

/* Function Prototypes
 */

/*----------------------------------------------------------------------*/

/* Exception Handling Functions
 */

extern dtF2exceptionPtr DTFFAR DTFCDCL dtF2setCritical(dtF2exceptionPtr func);
extern dtF2exceptionPtr DTFFAR DTFCDCL dtF2setError(dtF2exceptionPtr func);
extern dtF2exceptionPtr DTFFAR DTFCDCL dtF2setWarning(dtF2exceptionPtr func);

/*----------------------------------------------------------------------*/

/* Exception Handling Functions  Using Workspace
 */

extern dtF2exceptionPtr DTFFAR DTFCDCL dtF2setCriticalWS(dtF2wsHandle wsh,
   dtF2exceptionPtr func);
extern dtF2exceptionPtr DTFFAR DTFCDCL dtF2setErrorWS(dtF2wsHandle wsh,
   dtF2exceptionPtr func);
extern dtF2exceptionPtr DTFFAR DTFCDCL dtF2setWarningWS(dtF2wsHandle wsh,
   dtF2exceptionPtr func);

/*----------------------------------------------------------------------*/

/* Workspace Handling Functions
 */

extern dtF2wsHandle DTFFAR DTFCDCL dtF2getWS(void);
extern ErrType DTFFAR DTFCDCL dtF2releaseWS(dtF2wsHandle);
extern dtF2wsHandle DTFFAR DTFCDCL dtF2setDefaultWS(dtF2wsHandle);

/*----------------------------------------------------------------------*/

/* Login Functions
 */

extern ErrType DTFFAR DTFCDCL dtF2ident(MemPtr name, MemPtr passwd);

/*----------------------------------------------------------------------*/

/* Level 2 Main Service Functions
 */

extern ErrType DTFFAR DTFCDCL dtF2start(void);
extern ErrType DTFFAR DTFCDCL dtF2stop(void);
extern ErrType DTFFAR DTFCDCL dtF2exec(MemPtr sqlstring);
extern Boolean DTFFAR DTFCDCL dtF2resultAvailable(void);
extern ErrType DTFFAR DTFCDCL dtF2into(dtFfieldRange cols,
   VoidPtr valptr, Word valsize, ...);
extern ErrType DTFFAR DTFCDCL dtF2intoL(dtFfieldRange cols,
   VoidPtr valptr, Word valsize, dtFfieldLenPtr vallenptr, ...);
extern ErrType DTFFAR DTFCDCL dtF2getField(dtFfieldRange field,
   VoidPtr valptr, Word valsize, dtFfieldLenPtr vallenptr,
   dtFcolTypeRangePtr valtypeptr);
extern LongWord DTFFAR DTFCDCL dtF2rows(void);
extern dtFfieldRange DTFFAR DTFCDCL dtF2cols(void);
extern dtF2cinfoEntry DTFFAR DTFCDCL dtF2cinfo(dtFfieldRange colnr);
extern SignedLongWord DTFFAR DTFCDCL dtF2setMove(SignedLongWord);
extern ErrType DTFFAR DTFCDCL dtF2move(dtF2moveType, SignedLongWord);
extern LongWord DTFFAR DTFCDCL dtF2pos(void);

/*----------------------------------------------------------------------*/

/* Level 2 Main Service Functions Using Workspace
 */

extern ErrType DTFFAR DTFCDCL dtF2execWS(dtF2wsHandle wsh, MemPtr sqlstring);
extern Boolean DTFFAR DTFCDCL dtF2resultAvailableWS(dtF2wsHandle wsh);
extern ErrType DTFFAR DTFCDCL dtF2intoWS(dtF2wsHandle wsh,
   dtFfieldRange cols, VoidPtr valptr, Word valsize, ...);
extern ErrType DTFFAR DTFCDCL dtF2intoLWS(dtF2wsHandle wsh,
   dtFfieldRange cols,  VoidPtr valptr, Word valsize,
   dtFfieldLenPtr vallenptr, ...);
extern ErrType DTFFAR DTFCDCL dtF2getFieldWS(dtF2wsHandle wsh,
   dtFfieldRange field, VoidPtr valptr, Word valsize,
   dtFfieldLenPtr vallenptr, dtFcolTypeRangePtr valtypeptr);
extern LongWord DTFFAR DTFCDCL dtF2rowsWS(dtF2wsHandle wsh);
extern dtFfieldRange DTFFAR DTFCDCL dtF2colsWS(dtF2wsHandle wsh);
extern dtF2cinfoEntry DTFFAR DTFCDCL dtF2cinfoWS(dtF2wsHandle wsh,
   dtFfieldRange colnr);
extern SignedLongWord DTFFAR DTFCDCL dtF2setMoveWS(dtF2wsHandle wsh,
   SignedLongWord move);
extern ErrType DTFFAR DTFCDCL dtF2moveWS(dtF2wsHandle wsh,
  dtF2moveType type, SignedLongWord offset);
extern LongWord DTFFAR DTFCDCL dtF2posWS(dtF2wsHandle wsh);

/*----------------------------------------------------------------------*/

/* Configuration Functions
 */

extern SignedLongWord DTFFAR DTFCDCL dtF2setTimeOut(SignedLongWord timeout);
extern MemPtr DTFFAR DTFCDCL dtF2setMessageFile(MemPtr fname);
extern MemPtr DTFFAR DTFCDCL dtF2setSetupFile(MemPtr fname);

/*----------------------------------------------------------------------*/

/* Functions Provided As Macros 
 */

#define dtF2c(ptr) (MemPtr)(ptr), (Word)(sizeof(*ptr))
#define dtF2skip() (MemPtr)(BadMemPtr), (Word)(WordNoBits)
#define dtF2skipL() (MemPtr)(BadMemPtr), (Word)(WordNoBits), \
   (MemPtr)(BadMemPtr)

/*----------------------------------------------------------------------*/

/************************************************************************/

#if ( _CPLUSPLUS | _cplusplus | __CPLUSPLUS | __cplusplus )
}
#endif

/************************************************************************/

#endif

/************************************************************************/
