/************************************************************************/

/*
 *	Sample.c - dtF API Sample
 *	(C) Copyright 1990 - 1993 by sLAB Banzhaf & Lang GdbR.
 *	(C) Copyright 1990 - 1993 by theta group Denkel, Haerdle, Kern GdbR.
 *	All Rights Reserved.
 */

/*
 *	This sample performs the following steps
 *
 *	- Login to the dtF database
 *	- Create a relation named 'person'
 *	- Insert five records in the created relation 'person'
 *	- Request some records from the relation 'person' by using a dynamic
 	  sql statement
 *	- Scan retrieved data into C variables and print them
 */
 
/************************************************************************/

#pragma segment dtFSample

/****************************************************************************/
/* Identify your platform by defining constants                             */
/* (a better way to do this is to use the compiler directive '-d' in MPW-C  */
/* or define it in Options:Prefix when using Think-C                        */
/****************************************************************************/
#define DTFMAC		1
#define DTFSINGLE	1

/*************************************************/
/* Include dtF main headerfile.                  */
/* 'dtFHdr' reincludes all other necessary files */
/*************************************************/
#include "dtFHdr.h"

/**************************************/
/* Necessary include files for sample */
/**************************************/
#if ! THINKC
#include <Strings.h>
#endif

#include <string.h>
#include <StdLib.h>
#include <StdIO.h>
#include <StandardFile.h>

/**********************************************************************/
/* Include 'dtFCR.h'   						                 		  */
/* This is necessary if you want to create or recover dtF-databases!  */
/* (dtFQuery creates and recovers databases for you)                  */
/**********************************************************************/
#include "dtFCR.h"

/***********************/
/* Function prototypes */
/***********************/
void UserCommunication (char* theString);
void SendSQLStatement (MemPtr statement);
ErrType ErrorFunction (Word code, MemPtr group, MemPtr description, MemPtr String, Word Position);
ErrType ChooseDatabase (void);
ErrType CommunicateWithDatabase (void);
void main (void);

/************************************/
/* procedure for user communication */
/************************************/
void UserCommunication (char* theString) {
	printf (theString);
	fflush (stdout);
}

/***************************/
/* Sending a sql-statement */
/***************************/
void SendSQLStatement (MemPtr statement) {

	/********************************************/
	/* Use dtF2exec for all sql statements.     */
	/* Sql statements are simple ASCII strings  */
	/********************************************/
	dtF2exec (statement);
	
}

/*********************************************************************/
/* Standard error function for dtF errors in this sample program.    */
/* If you don't define your own error function, dtF will use         */
/* its default error function.                                       */
/* For more information about error handling please read the manual. */
/*********************************************************************/
ErrType ErrorFunction (Word code, MemPtr group, MemPtr description, MemPtr String, Word Position) {
	char	ErrorString[256];
	
	sprintf (ErrorString, "Code: %x Group: %s Desc.: %s", code, group, description);
	printf ("===============================================================\n");
	printf ("ERROR: %s\n", ErrorString);
	printf ("===============================================================\n");

	return (ErrBAD);
}


/*************************************/
/* Function to choose a database.    */
/*************************************/
ErrType ChooseDatabase () {

	/*********************************************/
	/* Variables for Mac's standard filehandling */
	/*********************************************/
	SFTypeList			typeList;
	StandardFileReply	reply;

	/**********************/
	/* User communication */
	/**********************/
	UserCommunication ("\nchoose a database\n");

	/**************************************************/
	/* The type of our example dtF database is 'DTFD' */
	/**************************************************/
	typeList[0] = 'DTFD';	

	/**********************************************/
	/* Open the standard Mac file selector dialog */
	/**********************************************/
	StandardGetFile (NULL, 1, &typeList, &reply);
	
	/************************/
	/* Test the result code */
	/************************/
	if (reply.sfGood) {
		
		/**********************************************/
		/* We need a C - string for the database name */
		/**********************************************/
		p2cstr (reply.sfFile.name);

		/***********************************************/
		/* Tell dtF about the location of the database */
		/***********************************************/
		dtFMacSetDatabasePath (reply.sfFile.vRefNum, 
						   reply.sfFile.parID,
						   (MemPtr) reply.sfFile.name
						   );
						   
		return (ErrOK);
		
	}
	else {
	
		return (ErrBAD);
	
	}
}

/**********************************************************************************/
/* Function to communicate with a dtF-database                                    */
/**********************************************************************************/
ErrType CommunicateWithDatabase (void)   {

	/*****************/
	/* Example value */
	/*****************/
	short myNumber = 10;
	
	/****************************/
	/* A buffer for sql strings */
	/****************************/
	char buffer[100];
	
	/********************************************************************/
	/* LongWord is a dtF datatype.                                      */
	/* 'records' contains the number of records in the dtF result table */
	/********************************************************************/
	LongWord records;
	
	/************************************************************************/
	/* I really don't know why, but the name of a loop variable must be 'i' */
	/************************************************************************/
	LongWord i;
	
	/*******************************/
	/* Variable for dtF error code */
	/*******************************/
	ErrType	erc;

	/***********************************/
	/* Buffers to store requested data */
	/***********************************/
	char buf [dtFmaxFieldLen+1]; 
	Word myId;
	char myName[251];
	
	/*************************************************************/
	/* Variables to get information about the data stored in dtF */
	/*************************************************************/
	dtFfieldLen fieldlen;
	dtFcolTypeRange restype;
	
	/**********************/
	/* User communication */
	/**********************/
	UserCommunication ("\ninitialize dtF error handling\n");
	
	/**********************************************************************/
	/* Tell the dtF error handler that we have our own error function     */
	/* (If you don't define your own error function dtF will use its      */
	/* default error function)                                            */
	/**********************************************************************/
	dtF2setWarning  (ErrorFunction);
	dtF2setError    (ErrorFunction);
	dtF2setCritical (ErrorFunction);

	/**********************/
	/* User communication */
	/**********************/
	UserCommunication ("identify user 'dtfadm' with password 'dtfadm'\n");

	/***********************************************************/
	/* Each user must login to get access to the dtF database. */
	/* Identify user "dtfadm" with password "dtfadm".          */
	/***********************************************************/
	dtF2ident ((MemPtr) "dtfadm", (MemPtr) "dtfadm");
	
	/**********************/						   
	/* User communication */
	/**********************/
	UserCommunication ("start dtF\n");

	/****************************************************/
	/* Connect to dtF server.                           */
	/* Implicitly start the first transaction.          */
	/* Check user and password at start of transaction. */
	/****************************************************/
	erc = dtF2start ();
			
	/*****************************************************************/
	/* If 'dtF2start' fails we must repair the database.             */
	/* These functions are implemented in the dtFrecover library.    */
	/* Interfaces are in the header 'dtFCR.h' 						 */
	/*****************************************************************/
	if (erc != ErrOK) {
	
		/**********************/						   
		/* User communication */
		/**********************/
		UserCommunication ("repairing the database ...");
		
		/***********************/
		/* repair the database */
		/***********************/
		erc = UUrecoverDatabase(BadMemPtr, false);
		
		/***********************/
		/* now start dtF again */
		/***********************/
		erc = dtF2start ();
		
	}

	/************************************************************************/
	/* If 'dtFstart' fails again, the database is completely damaged.       */
	/* This can happen if we have serious hardware errors on our hard disk. */ 
	/************************************************************************/
	if (erc == ErrOK) {
				
		/*****************************************************************/
		/* At this point we are logged into the server.                  */
		/* If errors occured they are handled by our error function.     */
		/*****************************************************************/
		
		/**********************/
		/* User communication */
		/**********************/
		UserCommunication ("create table 'person'\n");

		/*******************************************/
		/* Now we create a relation named 'person' */
		/*******************************************/
		SendSQLStatement ((MemPtr) "create table person (			\
										id			word,			\
										name		character(30),	\
										firstname	character(30),	\
										birthday	date,			\
										remark		shortstring		\
									   )"
						 );
					
									
		/**********************/
		/* User communication */
		/**********************/
		UserCommunication ("insert 5 records into table 'person'\n");

		/***********************/
		/* Insert some records */
		/***********************/
		SendSQLStatement ((MemPtr) "insert into person (id, name) values (1,  'Mickey')");
		SendSQLStatement ((MemPtr) "insert into person (id, name) values (20, 'Calvin')");
		SendSQLStatement ((MemPtr) "insert into person (id, name) values (14, 'Idefix')");
		SendSQLStatement ((MemPtr) "insert into person (id, name) values (41, 'Hobbes')");
		SendSQLStatement ((MemPtr) "insert into person (id, name) values (2,  'Bundy')");

		/**********************/
		/* User communication */
		/**********************/
		UserCommunication ("commit the transaction\n");

		/*****************************/
		/* We commit the transaction */
		/*****************************/
		SendSQLStatement ((MemPtr) "commit");

		/**************************************************************/
		/* After committing another transaction is started implicitly */
		/**************************************************************/
		
		/**********************/
		/* User communication */
		/**********************/
		UserCommunication ("retrieve all records with id greater than 10\n");
		
		/************************************************************************/
		/* We retrieve name and id for records with id greater than 'myNumber'. */
		/* You can create the sql statement using sUserCommunication                       */
		/************************************************************************/
		sprintf (buffer, "select name, id from person where id > %d", myNumber);
		SendSQLStatement ((MemPtr) buffer);
		
		/*****************************************/
		/* Now the result table is stored by dtF */
		/*****************************************/
		
		/***********************************************************************/
		/* We don't know how many records the result table contains            */
		/* so we ask dtF for it. Remember that we know our result table has    */
		/* only 2 columns (name, id). If you don't know the number of columns  */
		/* of a result table call 'dtF2cols'.                                  */
		/***********************************************************************/
		records = dtF2rows ();

		/***********************************************************************/
		/* We retrieve record by record, using the complicate way with         */
		/* manual cursor movement, to show you the functionality of cursor     */
		/* movement. The more intelligent way is to use 'dtF2into' wich uses   */
		/* automatic cursor movement (see manual)                              */
		/***********************************************************************/
		for (i = 0; i < records; i++) {
			
			/*****************************************************************/
			/* Move cursor, relative to the start of the result table,       */ 
			/* to the next record.                                           */
			/*****************************************************************/
			dtF2move (dtF2moveStart, i);
			
			/******************************************************************/
			/* Read the first field of this record into 'buf'.                */
			/* 'buf' has the maximum size of a dtF field, cause we know       */
			/* that this field is a string but not the length of this string. */
			/* The result is the length of this field ('fieldlen') and        */
			/* its dtF type ('restype')                                       */
			/******************************************************************/
			dtF2getField ((dtFfieldRange) 0, (MemPtr) buf,
						  sizeof (buf), &fieldlen, &restype);
			
			/******************************************************************/
			/* It there is a result convert the dtF-string into a C-string.   */
			/* (If you don't know the type of a field, you must evaluate      */
			/* the variable 'restype')                                        */
			/******************************************************************/
			if (fieldlen > 0) {
				dtFconvtToStr (restype, (MemPtr) myName, 
							   (MemPtr) buf, sizeof (buf), (Word) 0);
			}				
											
			/******************************************************************/
			/* Read the second field of this record into 'myId'.              */
			/* We know that this dtF field has the type 'word', so we have no */
			/* trouble with the buffersize.	     							  */
			/******************************************************************/
			dtF2getField ((dtFfieldRange) 1, (MemPtr) &myId,
						  sizeof (myId), &fieldlen, &restype);
											
			/************************/
			/* Show user the result */
			/************************/
			printf ("id -> %d    name -> %s\n", myId, myName);
			
		}


	}
	else {
		return (ErrBAD);
	}
	

	return (ErrOK);

}

void main (void) {
	
	/*********************/
	/* choose a database */
	/*********************/
	if (ChooseDatabase () != ErrOK) {
		UserCommunication ("\nNo database chosen !!!\n");
		UserCommunication ("Exit sample");
		return;
	}

	/**************************************/
	/* communicate with this dtF-database */
	/**************************************/
	if (CommunicateWithDatabase () != ErrOK) {
		UserCommunication ("Exit sample");
		return;			
	}	
		
	/**********************/
	/* User communication */
	/**********************/
	UserCommunication ("\n\nSample terminates successfully");
	
} /* cleanup is done automatically on exit by dtF */